"use strict";

this.name = "yah_gem_dice";
this.author = "spara";

//game track with bet multipliers
this.$track = [0, 0, 0, 4, 0, 8, 0, 17, 0, 35];

//add interface, if the player has started the game anywhere elsewhere than constore
this.startUpComplete = function() {
	if (!player.ship.dockedStation.hasRole("constore")) {
		this.shipExitedWitchspace();
	}
}

this.shipExitedWitchspace = function() {
	var constores = system.shipsWithRole("constore");
	if (constores.length > 0) {
		for (var i = 0; i < constores.length; i++) {
			constores[i].setInterface("yah_gem-dice",{
				title: "Smuggler's run",
				category: expandMissionText("yah_gem-gambling-category"),
				summary: expandMissionText("yah_gem-dice-summary"),
				callback: this.$intro.bind(this)
			});
		}
	}
}

//helper function for tabulating text on screen
this.$tabulate = function(line, width) {
	var hairSpace = String.fromCharCode(31);
	var space = defaultFont.measureString(" ");
	while (defaultFont.measureString(line) < width - space)
		line += " ";
	while (defaultFont.measureString(line) < width)
		line += hairSpace;
	return line;
}

//intro screen
this.$intro = function() {
	var bets = [1, 5, 10];
	//build and format the payoff table for screen
	var payoffMessage = "Bet:";
	for (var i = 0; i < 3; i++) {
		payoffMessage = this.$tabulate(payoffMessage, 5 * (i + 1));
		payoffMessage += bets[i] + " g";
	}
	var payoffMessage2 = "Score";
	for (var i = 0; i < 3; i++) {
		payoffMessage2 = this.$tabulate(payoffMessage2, 5 * (i + 1));
		payoffMessage2 += "Win!";
	}	
	payoffMessage = "\n\n" + payoffMessage + "\n\n" + payoffMessage2 + "\n";

	for (i = this.$track.length - 1; i > 0; i--) {
		if (this.$track[i] !== 0) {
			var line = this.$tabulate("", 0.5) + (i + 1);
			for (var j = 0; j < 3; j++) {
				line = this.$tabulate(line, 5 * (j + 1));
				line += this.$track[i] * bets[j] + " g";
			}
			payoffMessage += line+"\n";
		}
	}
	
	//format player credits for screen
	var credits = expandMissionText("yah_gem-credits")+player.ship.manifest["gem_stones"] + " g";
	
	var options = {
		"1_EXIT" : "Exit"
	};
	if (player.ship.manifest["gem_stones"] >= 1)
		options["2_10"] = "Bet 1";
	else credits += expandMissionText("yah_gem-no-credit");
	if (player.ship.manifest["gem_stones"] >= 5)
		options["3_50"] = "Bet 5";
	if (player.ship.manifest["gem_stones"] >= 10)
		options["4_100"] = "Bet 10";
	mission.runScreen({
		titleKey: "yah_gem-dice-title",
		screenID: "yah_gem_dice",
		message: expandMissionText("yah_gem-dice-intro-story")  + payoffMessage + credits,
		choices: options,
		background: "yah_gem_dice_bg-intro.png",
		exitScreen: "GUI_SCREEN_INTERFACES"
	},
	function (choice) {
		if (choice === "2_10") this.$playGame(1);
		else if (choice === "3_50") this.$playGame(5);
		else if (choice === "4_100") this.$playGame(10);
	}
	);
}

//da game
this.$playGame = function(bet, prevSum, round, track, select) {
	//init the game
	if (typeof round === 'undefined') {
		var round = 0;
		var track = this.$tabulate(" ", 3);
		player.ship.manifest["gem_stones"] -= bet;
	}
	
	//roll the dice
	var die1 = Math.ceil(6 * Math.random());
	var die2 = Math.ceil(6 * Math.random());
	var sum = die1 + die2;

	//check the guess
	var pass = true;
	if (typeof prevSum !== 'undefined') {
		if (select === "high_same") {
			if (sum < prevSum) 
				pass = false;
		}
		else if (select === "high") {
			if (sum <= prevSum) 
				pass = false;
		}
		else if (select === "low_same") {
			if (sum > prevSum) 
				pass = false;
		}
		else if (select === "low") {
			if (sum >= prevSum)
				pass = false;
		}
	}

//debug	
//pass = true; 

	//build the track display
	track += sum;
	//don't add tabs to the final score
	if (round < this.$track.length) {
		track = this.$tabulate(track, 3 + (round + 1) * 2.7);
	}
	var messageTrack = track;
	for (var i = round; i < this.$track.length; i++) {
		if (this.$track[i] !== 0) {
			messageTrack += bet * this.$track[i] + " g";
		}
		else messageTrack += "?";
		if (i < this.$track.length - 1) {
			messageTrack = this.$tabulate(messageTrack, 3 + (i + 2) * 2.7);
		}
	}

	var credits = "Gems: " + player.ship.manifest["gem_stones"] + " g";
	var betText = "Bet: " + bet + "\n";

	//correct guess
	if (pass) {
		var options = {};
		//cash out
		if (round > 0 && this.$track[round-1] !== 0) {
			var cash_out = bet * this.$track[round-1] + " g";
			options["1_CASH"] = "Cash out " + cash_out;
			var message = expandMissionText("yah_gem-dice-track-cash");
			var background = "yah_gem_dice_bg-cash.png";
		}
		else { 
			var message = expandMissionText("yah_gem-dice-track-basic");
			var background = "yah_gem_dice_bg-basic.png";
		}
		
		var background = "yah_gem_dice_bg-" + round + ".png"
		//guess choices
		if (round !== this.$track.length) {
			if (sum >= 7)
				options["2_HIGH_SAME"] = "Higher or same";
			else
				options["3_HIGH"] = "Higher";
			if (sum <= 7)
				options["4_LOW_SAME"] = "Lower or same";
			else
				options["5_LOW"] = "Lower";
		}
		//game finished
		else {
			var message = expandMissionText("yah_gem-dice-track-final");
		}
		mission.runScreen({
			titleKey: "yah_gem-dice-title",
			screenID: "yah_gem_dice",
			overlay: "yah_gem_dice_"+die1+"-"+die2+".png",
			message: "\n" + messageTrack + "\n\n" + message + "\n\n" + credits + "\n" + betText,
			choices: options,
			background: background
		}, 
		function (choice) {
			if (choice === "1_CASH") {
				var win = this.$track[round-1] * bet;
				player.ship.manifest["gem_stones"] += win;
				var notifySound = new SoundSource;
				notifySound.sound = "yah_gem_win.ogg";
				notifySound.play();
				this.$intro();
				return;
			}
			else if (choice === "2_HIGH_SAME") {
				this.$playGame(bet, sum, round + 1, track, "high_same");
				return;
			}
			else if (choice === "3_HIGH") {
				this.$playGame(bet, sum, round + 1, track, "high");
				return;
			}
			else if (choice === "4_LOW_SAME") {
				this.$playGame(bet, sum, round + 1, track, "low_same");
				return;
			}
			else if (choice === "5_LOW") {
				this.$playGame(bet, sum, round + 1, track, "low");
				return;
			}
		}
		);
	}
	else {
		var sound = "yah_gem_lose.ogg";
		var options = {
			"1_EXIT" : "Exit",
			"2_AGAIN" : "Change bet"
		};
		if (player.ship.manifest["gem_stones"] >= bet) options["3_PLAY"] = "Play again";
		mission.runScreen({
			titleKey: "yah_gem-dice-title",
			screenID: "yah_gem_dice",
			overlay: "yah_gem_dice_"+die1+"-"+die2+".png",
			message: "\n" + messageTrack + "\n\n" + expandMissionText("yah_gem-dice-track-fail") + "\n\n" + credits + "\n" + betText,
			choices: options,
			music: sound,
			background: "yah_gem_dice_bg-fail.png",
			exitScreen: "GUI_SCREEN_INTERFACES"
		},
		function (choice) {
			if (choice === "2_AGAIN") {
				this.$intro();
				return;
			}
			else if (choice === "3_PLAY") {
				this.$playGame(bet);
				return;
			}
		}
		);
	}
}
