"use strict";

this.name = "yah_gem_poker";
this.author = "spara";

this.payoffHands = ["Jacks or Better","Two Pair","Three of a Kind","Straight","Flush","Full House","Four of a Kind","Straight Flush","Royal Flush"];
this.payoffTable = [[1,2,3,4,6,9,25,50,250],[2,4,6,8,12,18,50,100,500],[3,6,9,12,18,27,75,150,750],[4,8,12,16,24,36,100,200,1000],[5,10,15,20,30,45,125,250,4000]];

//add interface, if the player has started the game anywhere elsewhere than constore
this.startUpComplete = function() {
	if (!player.ship.dockedStation.hasRole("constore")) {
		this.shipExitedWitchspace();
	}
}

this.shipExitedWitchspace = function() {
	var constores = system.shipsWithRole("constore");
	if (constores.length > 0) {
		for (var i = 0; i < constores.length; i++) {
			constores[i].setInterface("yah_gem-poker",{
				title: "Video poker",
				category: expandMissionText("yah_gem-gambling-category"),
				summary: expandMissionText("yah_gem-poker-summary"),
				callback: this.$intro.bind(this)
			});
		}
	}
}

//init a new deck
this.$virginDeck = function() {
	var deck = new Array();
	var suits = ["triangle", "crescent", "square", "star"];
	for (var i = 0; i < 4; i++) {
		for (var j = 1; j < 14; j++) {
			var card = new Object();
			card.suit = suits[i];
			card.rank = j;
			card.hold = 0; 
			deck.push(card);
		}
	}
	return deck;
}

//helper function for tabulating text on screen
this.$tabulate = function(line, width) {
	var hairSpace = String.fromCharCode(31);
	var space = defaultFont.measureString(" ");
	while (defaultFont.measureString(line) < width - space)
		line += " ";
	while (defaultFont.measureString(line) < width)
		line += hairSpace;
	return line;
}

//intro screen
this.$intro = function() {	
	//build and format the payoff table for screen
	var payoffMessage = this.$tabulate("Hand", 10);
	for (var i = 0; i < 5; i++) {
		payoffMessage += i + 1 + " g";
		payoffMessage = this.$tabulate(payoffMessage, 10 + 4 * (i + 1));
	}
	payoffMessage = "\n" + payoffMessage + "\n\n";
	for (i = 0; i < this.payoffHands.length; i++) {
		var line = this.payoffHands[this.payoffHands.length - i - 1];
		line = this.$tabulate(line, 10);
		for (var j = 0; j < 5; j++) {
			line = this.$tabulate(line, 10 + 4 * j);
			line += this.payoffTable[j][this.payoffHands.length - i - 1];
		}
		payoffMessage += line+"\n";
	}
	
	//format player credits for screen
	var credits = "Gems: " + player.ship.manifest["gem_stones"] + " g\n";
	
	//gambling choices
	var options = {
		"1_EXIT" : "Exit"
	};
	if (player.ship.manifest["gem_stones"] >= 1) {
		options["2_10"] = "Bet 1";
	}
	else payoffMessage += expandMissionText("yah_gem-no-credit");
	if (player.ship.manifest["gem_stones"] >= 2)
		options["3_20"] = "Bet 2";
	if (player.ship.manifest["gem_stones"] >= 3)
		options["4_30"] = "Bet 3";
	if (player.ship.manifest["gem_stones"] >= 4)
		options["5_40"] = "Bet 4";
	if (player.ship.manifest["gem_stones"] >= 5)
		options["6_50"] = "Bet 5";
	
	
	mission.runScreen({
		titleKey: "yah_gem-poker-title",
		message: this.$tabulate(expandMissionText("yah_gem-poker-intro-story"), 26) + credits + payoffMessage,
		model: "yah_gem_casino_poker_card_spin",
		background: "yah_gem_table_bg.png",
		choices: options,
		allowInterrupt: true,
		screenID: "yah_gem_poker",
		exitScreen: "GUI_SCREEN_INTERFACES"
	},
	function (choice) {
		if (choice === "2_10") this.$play(1);
		else if (choice === "3_20") this.$play(2);
		else if (choice === "4_30") this.$play(3);
		else if (choice === "5_40") this.$play(4);
		else if (choice === "6_50") this.$play(5);
	}
	);
}

//first pay, then deal
this.$play = function(bet) {
	player.ship.manifest["gem_stones"] -= bet;
	//init a new deck and draw a hand (don't show it yet)
	this.$playDeck = this.$virginDeck();
	var hand = new Array();
	for (var i = 0; i < 5; i++) hand.push(this.$drawCard());
	this.$deal(bet, hand);
}

//Deal
this.$deal = function(bet, hand, choice) {
	if (typeof choice === 'undefined') var choice = "0_HOLD";
	var options = {
		"0_HOLD" : "Hold/Cancel 1.",
		"1_HOLD" : "Hold/Cancel 2.",
		"2_HOLD" : "Hold/Cancel 3.",
		"3_HOLD" : "Hold/Cancel 4.",
		"4_HOLD" : "Hold/Cancel 5.",
		"5_DEAL" : "Draw"
	}
	var credits = "Gems: " + player.ship.manifest["gem_stones"] + " g\n";
	var betText = "Bet: " + bet + "\n";
	mission.runScreen({
		titleKey: "yah_gem-poker-title",
		message: this.$tabulate(expandMissionText("yah_gem-poker-deal-story"), 26) + credits + this.$tabulate(" ", 26) + betText,
		model: "yah_gem_casino_poker_card_base",
		background: "yah_gem_table_bg.png",
		spinModel: false,
		choices: options,
		screenID: "yah_gem_poker",
		initialChoicesKey: choice
	},
	function (choice) {
		if (choice === "0_HOLD")
			hand[0].hold = (hand[0].hold + 1) % 2;
		else if (choice === "1_HOLD")
			hand[1].hold = (hand[1].hold + 1) % 2;
		else if (choice === "2_HOLD")
			hand[2].hold = (hand[2].hold + 1) % 2;
		else if (choice === "3_HOLD")
			hand[3].hold = (hand[3].hold + 1) % 2;
		else if (choice === "4_HOLD")
			hand[4].hold = (hand[4].hold + 1) % 2;
		else if (choice === "5_DEAL") {
			this.$outCome(bet, hand);
			return;
		}
		this.$deal(bet, hand, choice);
		return;
	}
	);
	//update the cards on screen
	for (var i = 0; i < 5; i++)
		this.$changeCard(i, hand[i]);
}

//draw && outcome
this.$outCome = function(bet, hand) {
	//draw new cards
	for (var i = 0; i < 5; i++) {
		if (!hand[i].hold)
			hand[i] = this.$drawCard();
		else hand[i].hold = 0;
	}
	//check for wins
	var result = this.winCheck(hand);
	//fail
	if (result === -1) {
		var sound = "yah_gem_lose.ogg";
		var resultText = expandMissionText("yah_gem-poker-fail");
	}
	//win
	else {
		var win = this.payoffTable[(bet - 1)][result];
		var resultText = this.payoffHands[result] + ", you win " + win + " g of Gems.";
		player.ship.manifest["gem_stones"] += win;
		var sound = "yah_gem_win.ogg";
	}
	//format player credits for screen
	var credits = "Gems: " + player.ship.manifest["gem_stones"] + " g\n";
	var betText = "Bet: " + bet;
	var options = {
		"1_EXIT" : "Exit",
		"2_AGAIN" : "Change bet",
	};
	if (player.ship.manifest["gem_stones"] >= bet) options["3_DEAL"] = "Deal again";
	mission.runScreen({
		titleKey: "yah_gem-poker-title",
		message: this.$tabulate(resultText, 26) + credits + this.$tabulate(" ", 26) + betText,
		model: "yah_gem_casino_poker_card_base",
		background: "yah_gem_table_bg.png",
		spinModel: false,
		choices: options,
		music: sound,
		allowInterrupt: true,
		screenID: "yah_gem_poker",
		exitScreen: "GUI_SCREEN_INTERFACES"
	},
		function (choice) {
			if (choice === "2_AGAIN") {
				this.$intro();
				return;
			}
			else if (choice === "3_DEAL") {
				this.$play(bet);
				return;
			}
				
		}
	)
	//update the cards on screen
	for (var i = 0; i < 5; i++)
		this.$changeCard(i, hand[i]);
}

//draws a card from the current deck in play
this.$drawCard = function() {
	return this.$playDeck.splice(Math.floor(this.$playDeck.length * Math.random()),1)[0];
}

//update the texture and position for a card
this.$changeCard = function(place, card) {
	var fileName = "yah_gem_card_" + card.suit + "_" + card.rank + ".png";
	mission.displayModel.subEntities[place].setMaterials({"casino_poker_card_diffuse.png": {diffuse_map: fileName, emission_map: fileName, emission_modulate_color: "darkGrayColor"}});
	if (card.hold) {
		var position = mission.displayModel.subEntities[place].position;
		position[2] = 3;
		mission.displayModel.subEntities[place].position = position;
	}
}

//rules for a winning hand
this.winCheck = function(hand) {
	//work with a hand that has been ordered by rank
	var orderedHand = hand.concat();
	orderedHand.sort(function(a, b) {return a.rank - b.rank});
	
	//check for flush
	var suit = orderedHand[0].suit;
	var i = 0;
	var flush = false;
	for (i = 1; i < 5; i++) {
		if (orderedHand[i].suit !== suit) {
			i = -1; 
			break;
		}
	}
	//flush found
	if (i !== -1) flush = true;
	
	//check for straight
	var prevRank = orderedHand[0].rank;
	var straight = false
	for (i = 1; i < 5; i++) {
		if (orderedHand[i].rank - prevRank !== 1 && (i !== 1 || orderedHand[i].rank !== 10 || prevRank !== 1)) {
			i = -1; 
			break;
		}
		prevRank = orderedHand[i].rank;
	}

	//straight found
	if (i !== -1) straight = true;
	
	//straight flush
	if (flush && straight) {
		if (orderedHand[0].rank === 1 && orderedHand[4].rank === 13)
			return 8;//royal_flush
		else return 7;//straight_flush
	}
	
	//flush 
	if (flush) return 4;//flush"
	
	//straight
	if (straight) return 3;//straight
	
	//in the ordered list, for every adjacent two cards, check if they form a rank pair.
	prevRank = orderedHand[0].rank;
	var pairs = new Array();
	for (i = 1; i < 5; i++) {
		if (orderedHand[i].rank === prevRank) pairs.push(i);
		prevRank = orderedHand[i].rank;
	}

	//four of a kind (3 adjacent pairs)
	//full house (3 adjacent pairs)
	if (pairs.length === 3) {
		if (orderedHand[1].rank === orderedHand[2].rank && orderedHand[2].rank === orderedHand[3].rank)
			return 6;//four_of_a_kind
		else return 5;//full_house
	}

	//three of a kind (2 adjacent pairs)
	//two pair (2 adjacent pairs)
	if (pairs.length === 2) {
		if (pairs[1] - pairs[0] === 1)
			return 2;//three_of_a_kind
		else return 1;//two pair
	}
	
	//pair (jacks or better)
	if (pairs.length === 1) {
		if (orderedHand[pairs[0]].rank > 10 || orderedHand[pairs[0]].rank === 1)
			return 0;
	}
	
	return -1;
}
